@@ -1,16 +1,18 @@
-/* config.c - MemTest-86  Version 2.9
+/* config.c - MemTest-86  Version 3.0
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady, cbrady@sgi.com
  */
 #include "test.h"
 #include "screen_buffer.h"
+#include "controller.h"
 #define ITER 20
 
 extern int bail;
 extern struct tseq tseq[];
-extern short debug;
 extern struct vars *v;
+extern short e820_nr;
+extern char memsz_mode;
 
 char save[POP_H][POP_W];
 
@@ -18,21 +20,22 @@ void get_config()
 {
 	int flag = 0, sflag = 0, i, prt = 0;
         int reprint_screen = 0;
-	ulong addr;
+	ulong page;
 
 	popup();
 	wait_keyup();
 	while(!flag) {
-		cprint(POP_Y+1, POP_X+2, "Configuration:");
-		cprint(POP_Y+3, POP_X+6, "(1) Cache Mode");
-		cprint(POP_Y+4, POP_X+6, "(2) Test Selection");
-		cprint(POP_Y+5, POP_X+6, "(3) Address Range");
-		cprint(POP_Y+6, POP_X+6, "(4) Memory Sizing");
-		cprint(POP_Y+7, POP_X+6, "(5) Error Summary");
-		cprint(POP_Y+8, POP_X+6, "(6) Error Report Mode");
-		cprint(POP_Y+9, POP_X+6, "(7) Restart Test");
-		cprint(POP_Y+10, POP_X+6, "(9) Reprint Screen");
-		cprint(POP_Y+11,POP_X+6,"(0) Cancel");
+		cprint(POP_Y+1,  POP_X+2, "Configuration:");
+		cprint(POP_Y+3,  POP_X+6, "(1) Cache Mode");
+		cprint(POP_Y+4,  POP_X+6, "(2) Test Selection");
+		cprint(POP_Y+5,  POP_X+6, "(3) Address Range");
+		cprint(POP_Y+6,  POP_X+6, "(4) Memory Sizing");
+		cprint(POP_Y+7,  POP_X+6, "(5) Error Summary");
+		cprint(POP_Y+8,  POP_X+6, "(6) Error Report Mode");
+		cprint(POP_Y+9,  POP_X+6, "(7) ECC Mode"); 
+		cprint(POP_Y+10, POP_X+6, "(8) Restart Test");
+		cprint(POP_Y+11, POP_X+6, "(9) Reprint Screen");
+		cprint(POP_Y+12,POP_X+6,"(0) Cancel");
 
 		/* Wait for key release */
 		/* Fooey! This nuts'es up the serial input. */
@@ -145,7 +148,7 @@ void get_config()
 						"Test Selection:");
 					cprint(POP_Y+3, POP_X+4,
 						"Test Number [0-11]: ");
-					i = getval(POP_Y+3, POP_X+24);
+					i = getval(POP_Y+3, POP_X+24, 0);
 					if (i <= 11) {
 						v->testsel = i;
 					}
@@ -181,13 +184,12 @@ void get_config()
 						"Lower Limit: ");
 					cprint(POP_Y+4, POP_X+4,
 						"Current: ");
-					aprint(POP_Y+4, POP_X+13, v->lim_lower);
+					aprint(POP_Y+4, POP_X+13, v->plim_lower);
 					cprint(POP_Y+6, POP_X+4,
 						"New: ");
-					addr = getval(POP_Y+6, POP_X+9);
-					addr &= ~0x3ff;
-					if (addr+1024 <= v->lim_upper) {
-						v->lim_lower = addr;
+					page = getval(POP_Y+6, POP_X+9, 12);
+					if (page + 1 <= v->plim_upper) {
+						v->plim_lower = page;
 						bail++;
 					}
 					adj_mem();
@@ -201,13 +203,12 @@ void get_config()
 						"Upper Limit: ");
 					cprint(POP_Y+4, POP_X+4,
 						"Current: ");
-					aprint(POP_Y+4, POP_X+13, v->lim_upper);
+					aprint(POP_Y+4, POP_X+13, v->plim_upper);
 					cprint(POP_Y+6, POP_X+4,
 						"New: ");
-					addr = getval(POP_Y+6, POP_X+9);
-					addr &= ~0x3ff;
-					if (addr-1024 >= v->lim_lower) {
-						v->lim_upper = addr;
+					page = getval(POP_Y+6, POP_X+9, 12);
+					if  (page - 1 >= v->plim_lower) {
+						v->plim_upper = page;
 						bail++;
 					}
 					adj_mem();
@@ -216,9 +217,8 @@ void get_config()
 					break;
 				case 4:
 					/* All of memory */
-					v->lim_lower = 0;
-					v->lim_upper =
-					 (ulong)v->rmap[v->msegs-1].end;
+					v->plim_lower = 0;
+					v->plim_upper = v->pmap[v->msegs - 1].end;
 					bail++;
 					adj_mem();
 					find_ticks();
@@ -238,15 +238,15 @@ void get_config()
 			popclear();
 			cprint(POP_Y+1, POP_X+2, "Memory Sizing:");
 			cprint(POP_Y+3, POP_X+6, "(1) BIOS - Std");
-			if (v->e820_nr) {
+			if (e820_nr) {
 				cprint(POP_Y+4, POP_X+6, "(2) BIOS - All");
 				cprint(POP_Y+5, POP_X+6, "(3) Probe");
 				cprint(POP_Y+6, POP_X+6, "(0) Cancel");
-				cprint(POP_Y+2+v->memsz_mode, POP_X+5, ">");
+				cprint(POP_Y+2+memsz_mode, POP_X+5, ">");
 			} else {
 				cprint(POP_Y+4, POP_X+6, "(3) Probe");
 				cprint(POP_Y+5, POP_X+6, "(0) Cancel");
-				if (v->memsz_mode == SZ_MODE_BIOS) {
+				if (memsz_mode == SZ_MODE_BIOS) {
 					cprint(POP_Y+3, POP_X+5, ">");
 				} else {
 					cprint(POP_Y+4, POP_X+5, ">");
@@ -256,34 +256,43 @@ void get_config()
 			while (!sflag) {
 				switch(get_key()) {
 				case 2:
-					v->memsz_mode = SZ_MODE_BIOS;
+					memsz_mode = SZ_MODE_BIOS;
 					wait_keyup();
+					restart();
+/*
 					mem_size();
 					v->test = 0;
 					v->pass = 0;
 					v->total_ticks = 0;
 					bail++;
 					sflag++;
+*/
 					break;
 				case 3:
-					v->memsz_mode = SZ_MODE_BIOS_RES;
+					memsz_mode = SZ_MODE_BIOS_RES;
 					wait_keyup();
+					restart();
+/*
 					mem_size();
 					v->test = 0;
 					v->pass = 0;
 					v->total_ticks = 0;
 					bail++;
 					sflag++;
+*/
 					break;
 				case 4:
-					v->memsz_mode = SZ_MODE_PROBE;
+					memsz_mode = SZ_MODE_PROBE;
 					wait_keyup();
+					restart();
+/*
 					mem_size();
 					v->test = 0;
 					v->pass = 0;
 					v->total_ticks = 0;
 					bail++;
 					sflag++;
+*/
 					break;
 				case 11:
 				case 57:
@@ -314,7 +323,8 @@ void get_config()
 			cprint(POP_Y+1, POP_X+2, "Printing Mode:");
 			cprint(POP_Y+3, POP_X+6, "(1) Individual Errors");
 			cprint(POP_Y+4, POP_X+6, "(2) BadRAM Patterns");
-			cprint(POP_Y+5, POP_X+6, "(0) Cancel");
+			cprint(POP_Y+5, POP_X+6, "(3) Error Counts Only");
+			cprint(POP_Y+6, POP_X+6, "(0) Cancel");
 			cprint(POP_Y+3+v->printmode, POP_X+5, ">");
 			wait_keyup();
 			while (!sflag) {
@@ -331,6 +341,11 @@ void get_config()
 					sflag++;
 					prt++;
 					break;
+				case 4:
+					/* Error Counts Only */
+					v->printmode=PRINTMODE_NONE;
+					sflag++;
+					break;
 				case 11:
 				case 57:
 					/* 0/CR - Cancel */
@@ -341,6 +356,38 @@ void get_config()
 			popclear();
 			break;
 		case 8:
+			/* 7 - ECC Polling Mode */
+			popclear();
+			cprint(POP_Y+1, POP_X+2, "ECC Polling Mode:");
+			cprint(POP_Y+3, POP_X+6, "(1) Recommended");
+			cprint(POP_Y+4, POP_X+6, "(2) On");
+			cprint(POP_Y+5, POP_X+6, "(3) Off");
+			cprint(POP_Y+6, POP_X+6, "(0) Cancel");
+			wait_keyup();
+			while(!sflag) {
+				switch(get_key()) {
+				case 2:
+					set_ecc_polling(-1);
+					sflag++;
+					break;
+				case 3:
+					set_ecc_polling(1);
+					sflag++;
+					break;
+				case 4:
+					set_ecc_polling(0);
+					sflag++;
+					break;
+				case 11:
+				case 57:
+					/* 0/CR - Cancel */
+					sflag++;
+					break;
+				}
+			}
+			popclear();
+			break;
+		case 9:
 			wait_keyup();
 			restart();
 			break;
@@ -428,55 +475,45 @@ void clear_screen()
 	}
 }
 
-void adj_mem()
+void adj_mem(void)
 {
 	int i;
 
-	v->selected_mem = 0;
+	v->selected_pages = 0;
 	for (i=0; i< v->msegs; i++) {
 		/* Segment inside limits ? */
-		if ((ulong)v->rmap[i].start >= v->lim_lower &&
-				(ulong)v->rmap[i].end <= v->lim_upper) {
-			v->map[i].v = 1;
-			v->map[i].start = v->rmap[i].start;
-			v->map[i].end = v->rmap[i].end;
-			v->selected_mem += ((ulong)v->rmap[i].end -
-				(ulong)v->rmap[i].start);
+		if (v->pmap[i].start >= v->plim_lower &&
+				v->pmap[i].end <= v->plim_upper) {
+			v->selected_pages += (v->pmap[i].end - v->pmap[i].start);
 			continue;
 		}
 		/* Segment starts below limit? */
-		if ((ulong)v->rmap[i].start < v->lim_lower) {
+		if (v->pmap[i].start < v->plim_lower) {
 			/* Also ends below limit? */
-			if ((ulong)v->rmap[i].end < v->lim_lower) {
-				v->map[i].v = 0;
+			if (v->pmap[i].end < v->plim_lower) {
 				continue;
 			}
-			v->map[i].start=(ulong *)v->lim_lower;
 			
 			/* Ends past upper limit? */
-			if ((ulong)v->rmap[i].end > v->lim_upper) {
-				v->selected_mem += v->lim_upper - v->lim_lower;
-				v->map[i].end = (ulong *)v->lim_upper;
+			if (v->pmap[i].end > v->plim_upper) {
+				v->selected_pages += 
+					v->plim_upper - v->plim_lower;
 			} else {
 				/* Straddles lower limit */
-				v->selected_mem += ((ulong)v->rmap[i].end -
-					v->lim_lower);
+				v->selected_pages += 
+					(v->pmap[i].end - v->plim_lower);
 			}
-			v->map[i].v = 1;
 			continue;
 		}
 		/* Segment ends above limit? */
-		if ((ulong)v->rmap[i].end > v->lim_upper) {
+		if (v->pmap[i].end > v->plim_upper) {
 			/* Also starts above limit? */
-			if ((ulong)v->rmap[i].start > (ulong)v->lim_upper) {
-				v->map[i].v = 0;
+			if (v->pmap[i].start > v->plim_upper) {
 				continue;
 			}
 			/* Straddles upper limit */
-			v->map[i].end = (ulong *)v->lim_upper;
-			v->selected_mem += (v->lim_upper -
-				(ulong)v->rmap[i].start);
-			v->map[i].v = 1;
+			v->selected_pages += 
+				(v->plim_upper - v->pmap[i].start);
 		}
 	}
 }
