@@ -1,4 +1,4 @@
-/* config.h - MemTest-86  Version 2.9
+/* config.h - MemTest-86  Version 3.0
  *
  * Compile time configuration options
  *
@@ -26,8 +26,8 @@
 /*	Normally enabled */
 #define APM_OFF
 
-/* I8XX_USB_WAR - Enables a workaround for errors caused by BIOS USB keyboard */
-/*		  with Intel 810, 815 and 820 chipsets */
+/* USB_WAR - Enables a workaround for errors caused by BIOS USB keyboard */
+/*	and mouse support*/
 /*	Normally enabled */
-#define I8XX_USB_WAR
+#define USB_WAR
 
