@@ -1,36 +1,66 @@
-/* memsize.c - MemTest-86  Version 2.9
+/* memsize.c - MemTest-86  Version 3.0
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady, cbrady@sgi.com
  */
 
-#include <string.h>
 #include "test.h"
 #include "defs.h"
 #include "config.h"
 
+short e820_nr;
+short memsz_mode = SZ_MODE_BIOS;
+short firmware = FIRMWARE_UNKNOWN;
+
+static ulong alt_mem_k;
+static ulong ext_mem_k;
+static struct e820entry e820[E820MAX];
+
 extern ulong p1, p2;
 extern volatile ulong *p;
-extern struct vars *v;
 
-ulong m_lim = 0;
+static void sort_pmap(void);
+static int check_ram(void);
+static void memsize_bios(int res);
+static void memsize_820(int res);
+static void memsize_801(void);
+static int sanitize_e820_map(struct e820entry *orig_map, struct e820entry *new_bios,
+	short old_nr, short res);
+static void memsize_linuxbios();
+static void memsize_probe(void);
+static int check_ram(void);
 
 /*
  * Find out how much memory there is.
  */
-void mem_size()
+void mem_size(void)
 {
-	v->reserved = 0;
-	v->test_mem = 0;
+	int i;
+	v->reserved_pages = 0;
+	v->test_pages = 0;
 
-	switch (v->memsz_mode) {
+	/* On the first time thru only */
+	/* Make a copy of the memory info table so that we can re-evaluate */
+	/* The memory map later */
+	if (e820_nr == 0 && alt_mem_k == 0 && ext_mem_k == 0) {
+		ext_mem_k = mem_info.e88_mem_k;
+		alt_mem_k = mem_info.e801_mem_k;
+		e820_nr   = mem_info.e820_nr;
+		for (i=0; i< mem_info.e820_nr; i++) {
+			e820[i].addr = mem_info.e820[i].addr;
+			e820[i].size = mem_info.e820[i].size;
+			e820[i].type = mem_info.e820[i].type;
+		}
+	}
+
+	switch (memsz_mode) {
 	case SZ_MODE_BIOS:
 		/* Get the memory size from the BIOS */
-		memsize_820(0);
+		memsize_bios(0);
 		break;
 	case SZ_MODE_BIOS_RES:
 		/* Get the memory size from the BIOS, include reserved mem */
-		memsize_820(1);
+		memsize_bios(1);
 		break;
 	case SZ_MODE_PROBE:
 		/* Probe to find memory */
@@ -38,23 +68,77 @@ void mem_size()
 		cprint(LINE_INFO, COL_MMAP, " Probed ");
 		break;
 	}
-	v->lim_lower = 0;
-	v->lim_upper = (ulong)v->rmap[v->msegs-1].end;
-	v->rmap[0].start = (ulong *)(START_ADR);
+	/* Guarantee that pmap entries are in ascending order */
+	sort_pmap();
+	v->plim_lower = 0;
+	v->plim_upper = v->pmap[v->msegs-1].end;
 
 	adj_mem();
-	dprint(LINE_INFO, 19, v->reserved/1024, 7, 0);
+	aprint(LINE_INFO, COL_RESERVED, v->reserved_pages);
 }
 
-void memsize_820(int res)
+static void memsize_bios(int res)
+{
+	if (firmware == FIRMWARE_PCBIOS) {
+		memsize_820(res);
+	}
+	else if (firmware == FIRMWARE_LINUXBIOS) {
+		memsize_linuxbios();
+	}
+}
+
+static void sort_pmap(void)
+{
+	int i, j;
+	/* Do an insertion sort on the pmap, on an already sorted
+	 * list this should be a O(1) algorithm.
+	 */
+	for(i = 0; i < v->msegs; i++) {
+		/* Find where to insert the current element */
+		for(j = i -1; j >= 0; j--) {
+			if (v->pmap[i].start > v->pmap[j].start) {
+				j++;
+				break;
+			}
+		}
+		/* Insert the current element */
+		if (i != j) {
+			struct pmap temp;
+			temp = v->pmap[i];
+			memmove(&v->pmap[j], &v->pmap[j+1], 
+				(i -j)* sizeof(temp));
+			v->pmap[j] = temp;
+		}
+	}
+}
+static void memsize_linuxbios(void)
+{
+	int i, n;
+	/* Build the memory map for testing */
+	n = 0;
+	for (i=0; i < e820_nr; i++) {
+		unsigned long long end;
+		if (e820[i].type != E820_RAM) {
+			continue;
+		}
+		end = e820[i].addr;
+		end += e820[i].size;
+		v->pmap[n].start = (e820[i].addr + 4095) >> 12;
+		v->pmap[n].end = end >> 12;
+		v->test_pages += v->pmap[n].end - v->pmap[n].start;
+		n++;
+	}
+	v->msegs = n;
+	cprint(LINE_INFO, COL_MMAP, "LinuxBIOS");
+}
+static void memsize_820(int res)
 {
 	int i, n, nr;
-	ulong end;
 	struct e820entry nm[E820MAX];
 
 	/* Clean up, adjust and copy the BIOS-supplied E820-map. */
 	/* If the res arg is true reclassify reserved memory as E820_RAM */
-	nr = sanitize_e820_map(v->e820, nm, v->e820_nr, res);
+	nr = sanitize_e820_map(e820, nm, e820_nr, res);
 
 	/* If there is not a good 820 map use the BIOS 801/88 info */
 	if (nr < 1 || nr > E820MAX) {
@@ -65,35 +149,33 @@ void memsize_820(int res)
 	/* Build the memory map for testing */
 	n = 0;
 	for (i=0; i<nr; i++) {
-
-		/* Don't try to use any reserved memory that starts at the */
-		/* top of the address space. */
-		if (nm[i].type == E820_RAM && nm[i].addr < 0xff000000) {
+		if (nm[i].type == E820_RAM) {
+			unsigned long long start;
+			unsigned long long end;
+			start = nm[i].addr;
+			end = start + nm[i].size;
 
 			/* Don't ever use memory between 640 and 1024k */
-			if (nm[i].addr > RES_START && nm[n].addr < RES_END) {
-				if (nm[i].addr + nm[n].size < RES_END) {
+			if (start > RES_START && start < RES_END) {
+				if (end < RES_END) {
 					continue;
 				}
-				nm[n].size -= (RES_END - nm[i].addr);
-				nm[n].addr = RES_END;
+				start = RES_END;
 			}
-			end = nm[i].size + nm[i].addr;
-			if (end == 0) {
-				end = 0xfffffffc; }
 			if (end > RES_START && end < RES_END) {
-				nm[n].size -= (end - RES_START);
-				end = nm[i].size + nm[i].addr;
+				end = RES_START;
 			}
-			v->rmap[n].start = (ulong *)nm[i].addr;
-			v->rmap[n].end = (ulong *)end;
-			v->test_mem += (end - nm[i].addr);
+			v->pmap[n].start = (start + 4095) >> 12;
+			v->pmap[n].end = end >> 12;
+			v->test_pages += v->pmap[n].end - v->pmap[n].start;
 			n++;
 		} else {
-			/* If this is the top of memory then it's not */
-			/* real ram so don't confuse things by counting it. */
+			/* If this is reserved memory starting at the top
+			 * of memory then don't count it as reserved, since
+			 * it is very unlikely to be real memory.
+			 */
 			if (nm[i].addr < 0xff000000) {
-				v->reserved += nm[i].size;
+				v->reserved_pages += nm[i].size >> 12;
 			}
 		}
 	}
@@ -105,29 +187,29 @@ void memsize_820(int res)
 	}
 }
 	
-void memsize_801()
+static void memsize_801(void)
 {
 	ulong mem_size;
 
 	/* compare results from 88 and 801 methods and take the greater */
 	/* These sizes are for extended memory in 1k units. */
 
-	if (v->alt_mem_k < v->ext_mem_k) {
-		mem_size = v->ext_mem_k;
+	if (alt_mem_k < ext_mem_k) {
+		mem_size = ext_mem_k;
 		cprint(LINE_INFO, COL_MMAP, "e88-Std ");
 	} else {
-		mem_size = v->alt_mem_k;
+		mem_size = alt_mem_k;
 		cprint(LINE_INFO, COL_MMAP, "e801-Std");
 	}
 	/* First we map in the first 640k */
-	v->rmap[0].start = 0;
-	v->rmap[0].end = (ulong *)RES_START;
-	v->test_mem = RES_START;
+	v->pmap[0].start = 0;
+	v->pmap[0].end = RES_START >> 12;
+	v->test_pages = RES_START >> 12;
 
 	/* Now the extended memory */
-	v->rmap[1].start = (ulong *)RES_END;
-	v->rmap[1].end = (ulong *)((mem_size+1024)*1024);
-	v->test_mem += mem_size * 1024;
+	v->pmap[1].start = (RES_END + 4095) >> 12;
+	v->pmap[1].end = (mem_size + 1024) >> 2;
+	v->test_pages += mem_size >> 2;
 	v->msegs = 2;
 }
 
@@ -138,7 +220,7 @@ void memsize_801()
  * replaces the original e820 map with a new one, removing overlaps.
  *
  */
-int sanitize_e820_map(struct e820entry *orig_map, struct e820entry *new_bios,
+static int sanitize_e820_map(struct e820entry *orig_map, struct e820entry *new_bios,
 	short old_nr, short res)
 {
 	struct change_member {
@@ -202,11 +284,18 @@ int sanitize_e820_map(struct e820entry *orig_map, struct e820entry *new_bios,
 	for (i=0; i<old_nr; i++) {
 		if (biosmap[i].addr + biosmap[i].size < biosmap[i].addr)
 			return 0;
-		/* If we want to test the reserved memory include everything */
 		if (res) {
+			/* If we want to test the reserved memory include
+			 * everything except for reserved segments that start
+			 * at the  the top of memory
+			 */
+			if (biosmap[i].type == E820_RESERVED &&
+					biosmap[i].addr > 0xff000000) {
+				continue;
+			}
 			biosmap[i].type = E820_RAM;
 		} else {
-			/* It should always be safe to test ACPI ram */
+			/* It is always be safe to test ACPI ram */
 			if ( biosmap[i].type == E820_ACPI) {
 				biosmap[i].type = E820_RAM;
 			}
@@ -298,15 +387,16 @@ int sanitize_e820_map(struct e820entry *orig_map, struct e820entry *new_bios,
 	return(new_bios_entry);
 }
 
-void memsize_probe()
+static void memsize_probe(void)
 {
 	int i, n;
+	ulong m_lim;
+	static unsigned long magic = 0x1234569;
 
 	/* Since all address bits may not be decoded, the search for memory
 	 * must be limited.  The max address is found by checking for
 	 * memory wrap from 1MB to 4GB.  */
-	v->rmap[0].start = (ulong *)0x1234569;
-	p1 = (ulong)&v->rmap[0].start;
+	p1 = (ulong)&magic;
 	m_lim = 0xfffffffc; 
 	for (p2 = 0x100000; p2; p2 <<= 1) {  
 		p = (ulong *)(p1 + p2);
@@ -321,42 +411,38 @@ void memsize_probe()
 
 	/* Find all segments of RAM */
 
-	p = (ulong *)(START_ADR);
 	i = 0;
-	v->rmap[i].start = 0;
+	v->pmap[i].start = ((ulong)&_end + (1 << 12) - 1) >> 12;
+	p = (ulong *)(v->pmap[i].start << 12);
 
 	/* Limit search for memory to m_lim and make sure we don't 
 	 * overflow the 32 bit size of p.  */
-	while ((ulong)p < m_lim && (ulong)p >= START_ADR) {
+	while ((ulong)p < m_lim && (ulong)p >= (ulong)&_end) {
 		/*
 		 * Skip over reserved memory
 		 */
-		if ((ulong)p < RES_END &&
-				(ulong)p >= RES_START) {
-			v->rmap[i].end = (ulong *)RES_START;
-			v->rmap[i].v = 1;
-			v->test_mem += (v->rmap[i].end - v->rmap[i].start);
+		if ((ulong)p < RES_END && (ulong)p >= RES_START) {
+			v->pmap[i].end = RES_START >> 12;
+			v->test_pages += (v->pmap[i].end - v->pmap[i].start);
 			p = (ulong *)RES_END;
 			i++;
-			v->rmap[i].start = 0;
+			v->pmap[i].start = 0;
 			goto fstart;
 		}
 
 		if (check_ram() == 0) {
 			/* ROM or nothing at this address, record end addrs */
-			v->rmap[i].end = p;
-			v->rmap[i].v = 1;
-			v->test_mem += (v->rmap[i].end - v->rmap[i].start);
+			v->pmap[i].end = ((ulong)p) >> 12;
+			v->test_pages += (v->pmap[i].end - v->pmap[i].start);
 			i++;
-			v->rmap[i].start = 0;
+			v->pmap[i].start = 0;
 fstart:
 
 			/* We get here when there is a gap in memory.
 			 * Loop until we find more ram, the gap is more
 			 * than 32768k or we hit m_lim */
-			n = 32768;
-			while ((ulong)p < m_lim &&
-					(ulong)p >= START_ADR) {
+			n = 32768 >> 2;
+			while ((ulong)p < m_lim && (ulong)p >= (ulong)&_end) {
 
 				/* Skip over video memory */
 				if ((ulong)p < RES_END &&
@@ -365,7 +451,7 @@ fstart:
 				}
 				if (check_ram() == 1) {
 					/* More RAM, record start addrs */
-					v->rmap[i].start = p;
+					v->pmap[i].start = (ulong)p >> 12;
 					break;
 				}
 
@@ -375,28 +461,20 @@ fstart:
 					p = (ulong *)m_lim;
 					break;
 				}
-				p += 0x400;
+				p += 0x1000;
 			}
 		}
-		p += 0x400;
+		p += 0x1000;
 	}
 
 	/* If there is ram right up to the memory limit this will record
 	 * the last address.  */
-	if (v->rmap[i].start) {
-		v->rmap[i].end = (ulong *)m_lim;
-		v->rmap[i].v = 1;
-		v->test_mem += (v->rmap[i].end - v->rmap[i].start);
+	if (v->pmap[i].start) {
+		v->pmap[i].end = m_lim >> 12;
+		v->test_pages += (v->pmap[i].end - v->pmap[i].start);
 		i++;
 	}
-/*
-v->rmap[i].start = 0xd0000000;
-v->rmap[i].end = 0xd0001000;
-v->test_mem += (v->rmap[i].end - v->rmap[i].start);
-i++;
-*/
 	v->msegs = i;
-	v->test_mem *= 4;
 }
 
 /* check_ram - Determine if this address points to memory by checking
@@ -404,7 +482,8 @@ i++;
  * We then check that at least one bit changed in each byte before
  * believing that it really is memory.  */
 
-int check_ram() {
+static int check_ram(void) 
+{
         int s;
 
         p1 = *p;
