@@ -1,4 +1,4 @@
-/* test.c - MemTest-86  Version 2.9
+/* test.c - MemTest-86  Version 3.0
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady, cbrady@sgi.com
@@ -12,10 +12,17 @@ extern ulong p1, p2;
 extern struct vars *v;
 extern int test_ticks, nticks;
 extern struct tseq tseq[];
+void poll_errors();
 
-int header = 0;
 int ecount = 0;
 
+static void update_err_counts(void);
+static void print_err_counts(void);
+
+static inline ulong roundup(ulong value, ulong mask)
+{
+	return (value + mask) & ~mask;
+}
 /*
  * Memory address test, walking ones
  */
@@ -28,57 +35,54 @@ void addr_tst1()
 
 	/* Test the global address bits */
 	for (p1=0, j=0; j<2; j++) {
-        	hprint(LINE_INFO, COL_PAT, p1);
+        	hprint(LINE_PAT, COL_PAT, p1);
 
-		/* Set pattern in address 0x40000 */
-		p = (ulong *)0x40000;
+		/* Set pattern in our lowest multiple of 0x20000 */
+		p = (ulong *)roundup((ulong)v->map[0].start, 0x1ffff);
 		*p = p1;
 	
-		/* Now write pattern complment */
+		/* Now write pattern compliment */
 		p1 = ~p1;
 		end = v->map[segs-1].end;
 		for (i=0; i<1000; i++) {
 			mask = 4;
-			while (mask < (ulong)end && mask) {
+			do {
 				pt = (ulong *)((ulong)p | mask);
 				if (pt == p) {
 					mask = mask << 1;
 					continue;
 				}
+				if (pt >= end) {
+					break;
+				}
 				*pt = p1;
 				if ((bad = *p) != ~p1) {
-					ad_err1((ulong)p,
-					(ulong)mask, bad, ~p1);
+					ad_err1((ulong *)p, (ulong *)mask,
+						bad, ~p1);
 					i = 1000;
 				}
 				mask = mask << 1;
-			}
+			} while(mask);
 		}
-		BAILR
 		do_tick();
-		
+		BAILR
 	}
 
 	/* Now check the address bits in each bank */
 	/* If we have more than 8mb of memory then the bank size must be */
 	/* bigger than 256k.  If so use 1mb for the bank size. */
-	if (v->lim_upper > 0x800000) {
+	if (v->pmap[v->msegs - 1].end > (0x800000 >> 12)) {
 		bank = 0x100000;
 	} else {
 		bank = 0x40000;
 	}
 	for (p1=0, k=0; k<2; k++) {
-        	hprint(LINE_INFO, COL_PAT, p1);
+        	hprint(LINE_PAT, COL_PAT, p1);
 
 		for (j=0; j<segs; j++) {
-			if (!v->map[j].v) {
-				continue;
-			}
-			p = (ulong *)v->map[j].start;
+			p = v->map[j].start;
 			/* Force start address to be a multiple of 256k */
-			if (p && p < (ulong *)bank) {
-				p = (ulong *)bank;
-			}
+			p = (ulong *)roundup((ulong)p, bank - 1);
 			end = v->map[j].end;
 			while (p < end) {
 				*p = p1;
@@ -86,30 +90,37 @@ void addr_tst1()
 				p1 = ~p1;
 				for (i=0; i<200; i++) {
 					mask = 4;
-					while (mask < (ulong)end &&
-							mask) {
+					do {
 						pt = (ulong *)
 						    ((ulong)p | mask);
 						if (pt == p) {
 							mask = mask << 1;
 							continue;
 						}
+						if (pt >= end) {
+							break;
+						}
 						*pt = p1;
 						if ((bad = *p) != ~p1) {
-							ad_err1((ulong)p,
-							  (ulong)mask, bad,~p1);
+							ad_err1((ulong *)p,
+							    (ulong *)mask,
+							    bad,~p1);
 							i = 200;
 						}
 						mask = mask << 1;
-					}
+					} while(mask);
+				}
+				if (p + bank > p) {
+					p += bank;
+				} else {
+					p = end;
 				}
-				BAILR
-				p += bank;
 				p1 = ~p1;
 			}
 		}
-		p1 = ~p1;
 		do_tick();
+		BAILR
+		p1 = ~p1;
 	}
 }
 
@@ -122,13 +133,10 @@ void addr_tst2()
 	volatile ulong *pe;
 	volatile ulong *end, *start;
 
-        cprint(LINE_INFO, COL_PAT, "        ");
+        cprint(LINE_PAT, COL_PAT, "        ");
 
 	/* Write each address with it's own address */
 	for (j=0; j<segs; j++) {
-		if (!v->map[j].v) {
-			continue;
-		}
 		start = v->map[j].start;
 		end = v->map[j].end;
 		pe = (ulong *)start;
@@ -173,9 +181,6 @@ void addr_tst2()
 
 	/* Each address should have its own address */
 	for (j=0; j<segs; j++) {
-		if (!v->map[j].v) {
-			continue;
-		}
 		start = v->map[j].start;
 		end = v->map[j].end;
 		pe = (ulong *)start;
@@ -249,16 +254,13 @@ void movinv1(int iter, ulong p1, ulong p2)
 	volatile ulong *start,*end;
 
 	/* Display the current pattern */
-        hprint(LINE_INFO, COL_PAT, p1);
+        hprint(LINE_PAT, COL_PAT, p1);
 
 	/* Initialize memory with the initial pattern.  */
 	for (j=0; j<segs; j++) {
-		if (!v->map[j].v) {
-			continue;
-		}
 		start = v->map[j].start;
 		end = v->map[j].end;
-		pe = (ulong *)start;
+		pe = start;
 		p = start;
 		done = 0;
 		do {
@@ -268,7 +270,7 @@ void movinv1(int iter, ulong p1, ulong p2)
 			} else {
 				pe = end;
 			}
-			if (pe > end) {
+			if (pe >= end) {
 				pe = end;
 				done++;
 			}
@@ -284,8 +286,8 @@ void movinv1(int iter, ulong p1, ulong p2)
 			asm __volatile__ (
 				"rep\n\t" \
 				"stosl\n\t"
-				"movl %%edi,p\n\t" \
-				:: "c" (len), "D" (p), "a" (p1)
+				: "=D" (p)
+				: "c" (len), "0" (p), "a" (p1)
 			);
 			do_tick();
 			BAILR
@@ -297,12 +299,9 @@ void movinv1(int iter, ulong p1, ulong p2)
 	 * up and then from the top down.  */
 	for (i=0; i<iter; i++) {
 		for (j=0; j<segs; j++) {
-			if (!v->map[j].v) {
-				continue;
-			}
 			start = v->map[j].start;
 			end = v->map[j].end;
-			pe = (ulong *)start;
+			pe = start;
 			p = start;
 			done = 0;
 			do {
@@ -357,8 +356,8 @@ void movinv1(int iter, ulong p1, ulong p2)
 					"jmp L5\n" \
 
 					"L4:\n\t" \
-					"mov %%edi,p\n\t" \
-					:: "a" (p1), "D" (p), "d" (pe), "b" (p2)
+					: "=D" (p)
+					: "a" (p1), "0" (p), "d" (pe), "b" (p2)
 					: "ecx"
 				);
 				do_tick();
@@ -366,13 +365,10 @@ void movinv1(int iter, ulong p1, ulong p2)
 			} while (!done);
 		}
 		for (j=segs-1; j>=0; j--) {
-			if (!v->map[j].v) {
-				continue;
-			}
 			start = v->map[j].start;
 			end = v->map[j].end;
-			pe = end;
-			p = --end;
+			pe = end -1;
+			p = end -1;
 			done = 0;
 			do {
 				/* Check for underflow */
@@ -397,37 +393,39 @@ void movinv1(int iter, ulong p1, ulong p2)
  *				} while (p-- > pe);
  */
 				asm __volatile__ (
-					"jmp L9\n\t" \
+					"addl $4, %%edi\n\t"
+					"jmp L9\n\t"
 
-					".p2align 4,,7\n\t" \
-					"L9:\n\t" \
-					"movl (%%edi),%%ecx\n\t" \
-					"cmpl %%ebx,%%ecx\n\t" \
-					"jne L6\n\t" \
-					"L10:\n\t" \
-					"movl %%eax,(%%edi)\n\t" \
-					"subl $4,%%edi\n\t" \
-					"cmpl %%edi,%%edx\n\t" \
-					"jle L9\n\t" \
-					"jmp L7\n" \
+					".p2align 4,,7\n\t"
+					"L9:\n\t"
+					"subl $4, %%edi\n\t"
+					"movl (%%edi),%%ecx\n\t"
+					"cmpl %%ebx,%%ecx\n\t"
+					"jne L6\n\t"
+					"L10:\n\t"
+					"movl %%eax,(%%edi)\n\t"
+					"cmpl %%edi, %%edx\n\t"
+					"jne L9\n\t"
+					"subl $4, %%edi\n\t"
+					"jmp L7\n\t"
 
-					"L6:\n\t" \
-					"pushl %%edx\n\t" \
-					"pushl %%eax\n\t" \
-					"pushl %%ecx\n\t" \
-					"pushl %%ebx\n\t" \
-					"pushl %%edi\n\t" \
-					"call error\n\t" \
-					"popl %%edi\n\t" \
-					"popl %%ebx\n\t" \
-					"popl %%ecx\n\t" \
-					"popl %%eax\n\t" \
-					"popl %%edx\n\t" \
-					"jmp L10\n" \
+					"L6:\n\t"
+					"pushl %%edx\n\t"
+					"pushl %%eax\n\t"
+					"pushl %%ecx\n\t"
+					"pushl %%ebx\n\t"
+					"pushl %%edi\n\t"
+					"call error\n\t"
+					"popl %%edi\n\t"
+					"popl %%ebx\n\t"
+					"popl %%ecx\n\t"
+					"popl %%eax\n\t"
+					"popl %%edx\n\t"
+					"jmp L10\n"
 
-					"L7:\n\t" \
-					"mov %%edi,p\n\t" \
-					:: "a" (p1), "D" (p), "d" (pe), "b" (p2)
+					"L7:\n\t"
+					: "=D" (p)
+					: "a" (p1), "0" (p), "d" (pe), "b" (p2)
 					: "ecx"
 				);
 				do_tick();
@@ -447,16 +445,13 @@ void movinv32(int iter, ulong p1, ulong lb, ulong hb, int sval, int off)
 	p3 = sval << 31;
 
 	/* Display the current pattern */
-        hprint(LINE_INFO, COL_PAT, p1);
+        hprint(LINE_PAT, COL_PAT, p1);
 
 	/* Initialize memory with the initial pattern.  */
 	for (j=0; j<segs; j++) {
-		if (!v->map[j].v) {
-			continue;
-		}
 		start = v->map[j].start;
 		end = v->map[j].end;
-		pe = (ulong *)start;
+		pe = start;
 		p = start;
 		done = 0;
 		k = off;
@@ -522,9 +517,6 @@ void movinv32(int iter, ulong p1, ulong lb, ulong hb, int sval, int off)
 	 * up and then from the top down.  */
 	for (i=0; i<iter; i++) {
 		for (j=0; j<segs; j++) {
-			if (!v->map[j].v) {
-				continue;
-			}
 			start = v->map[j].start;
 			end = v->map[j].end;
 			pe = start;
@@ -632,13 +624,10 @@ void movinv32(int iter, ulong p1, ulong lb, ulong hb, int sval, int off)
 		}
 		k++;
 		for (j=segs-1; j>=0; j--) {
-			if (!v->map[j].v) {
-				continue;
-			}
 			start = v->map[j].start;
 			end = v->map[j].end;
-			p = --end;
-			pe = end;
+			p = end -1;
+			pe = end -1;
 			done = 0;
 			do {
 				/* Check for underflow */
@@ -741,15 +730,12 @@ void modtst(int offset, int iter, ulong p1, ulong p2)
 	volatile ulong *start, *end;
 
 	/* Display the current pattern */
-        hprint(LINE_INFO, COL_PAT-2, p1);
-	cprint(LINE_INFO, COL_PAT+6, "-");
-        dprint(LINE_INFO, COL_PAT+7, offset, 2, 1);
+        hprint(LINE_PAT, COL_PAT-2, p1);
+	cprint(LINE_PAT, COL_PAT+6, "-");
+        dprint(LINE_PAT, COL_PAT+7, offset, 2, 1);
 
 	/* Write every nth location with pattern */
 	for (j=0; j<segs; j++) {
-		if (!v->map[j].v) {
-			continue;
-		}
 		start = v->map[j].start;
 		end = v->map[j].end;
 		pe = (ulong *)start;
@@ -794,9 +780,6 @@ void modtst(int offset, int iter, ulong p1, ulong p2)
 	/* Write the rest of memory "iter" times with the pattern complement */
 	for (l=0; l<iter; l++) {
 		for (j=0; j<segs; j++) {
-			if (!v->map[j].v) {
-				continue;
-			}
 			start = v->map[j].start;
 			end = v->map[j].end;
 			pe = (ulong *)start;
@@ -856,9 +839,6 @@ void modtst(int offset, int iter, ulong p1, ulong p2)
 
 	/* Now check every nth location */
 	for (j=0; j<segs; j++) {
-		if (!v->map[j].v) {
-			continue;
-		}
 		start = v->map[j].start;
 		end = v->map[j].end;
 		pe = (ulong *)start;
@@ -933,17 +913,14 @@ void block_move(int iter)
 	volatile ulong p, pe, pp;
 	volatile ulong start, end;
 
-        cprint(LINE_INFO, COL_PAT-2, "          ");
+        cprint(LINE_PAT, COL_PAT-2, "          ");
 
 	/* Initialize memory with the initial pattern.  */
 	for (j=0; j<segs; j++) {
-		if (!v->map[j].v) {
-			continue;
-		}
 		start = (ulong)v->map[j].start;
-#ifdef I8XX_USB_WAR
+#ifdef USB_WAR
 		/* We can't do the block move test on low memory beacuase
-		 * BIOS USB support clobbers location 0x4e0
+		 * BIOS USB support clobbers location 0x410 and 0x4e0
 		 */
 		if (start < 0x4f0) {
 			start = 0x4f0;
@@ -1009,13 +986,10 @@ void block_move(int iter)
 	 * Then move the data to the original location + 32 bytes
 	 */
 	for (j=0; j<segs; j++) {
-		if (!v->map[j].v) {
-			continue;
-		}
 		start = (ulong)v->map[j].start;
-#ifdef I8XX_USB_WAR
+#ifdef USB_WAR
 		/* We can't do the block move test on low memory beacuase
-		 * BIOS USB support clobbers location 0x4e0
+		 * BIOS USB support clobbers location 0x410 and 0x4e0
 		 */
 		if (start < 0x4f0) {
 			start = 0x4f0;
@@ -1079,13 +1053,10 @@ void block_move(int iter)
 	 * adjacent words are the same.
 	 */
 	for (j=0; j<segs; j++) {
-		if (!v->map[j].v) {
-			continue;
-		}
 		start = (ulong)v->map[j].start;
-#ifdef I8XX_USB_WAR
+#ifdef USB_WAR
 		/* We can't do the block move test on low memory beacuase
-		 * BIOS USB support clobbers location 0x4e0
+		 * BIOS USB support clobbers location 0x4e0 and 0x410
 		 */
 		if (start < 0x4f0) {
 			start = 0x4f0;
@@ -1154,7 +1125,7 @@ void error(ulong *adr, ulong good, ulong bad)
 	int patnchg;
 
 	xor = good ^ bad;
-#ifdef I8XX_USB_WAR
+#ifdef USB_WAR
 	/* Skip any errrors that appear to be due to the BIOS using location
 	 * 0x4e0 for USB keyboard support.  This often happens with Intel
          * 810, 815 and 820 chipsets.  It is possible that we will skip
@@ -1168,21 +1139,19 @@ void error(ulong *adr, ulong good, ulong bad)
 	/* Process the address in the pattern administration */
 	patnchg=insertaddress ((ulong) adr);
 
+	update_err_counts();
 	if (v->printmode == PRINTMODE_ADDRESSES) {
 
 		/* Don't display duplicate errors */
 		if ((ulong)adr == (ulong)v->eadr && xor == v->exor) {
-			dprint(LINE_INFO, COL_ERR, ++(v->ecount), 6, 0);
+			print_err_counts();
 			dprint(v->msg_line, 62, ++ecount, 5, 0);
 			return;
 		}
 		print_err(adr, good, bad, xor);
 
-	} else {
-		/* printmode == PRINTMODE_PATTERNS */
-
-		dprint(LINE_INFO, COL_ERR, ++(v->ecount), 6, 0);
-		tseq[v->test].errors++;
+	} else if (v->printmode == PRINTMODE_PATTERNS) {
+		print_err_counts();
 
 		if (patnchg) { 
 			printpatn();
@@ -1199,6 +1168,10 @@ void mv_error(ulong *adr, ulong good, ulong bad)
 {
 	ulong xor;
 
+	update_err_counts();
+	if (v->printmode == PRINTMODE_NONE) {
+		return;
+	}
 	xor = good ^ bad;
 	print_err(adr, good, bad, xor);
 }
@@ -1208,9 +1181,15 @@ void mv_error(ulong *adr, ulong good, ulong bad)
  * Since this is strictly an address test, trying to create BadRAM
  * patterns does not make sense.  Just report the error.
  */
-void ad_err1(ulong adr1, ulong adr2, ulong good, ulong bad)
+void ad_err1(ulong *adr1, ulong *adr2, ulong good, ulong bad)
 {
-	print_err(adr1, good, bad, adr1 ^ adr2);
+	ulong xor;
+	update_err_counts();
+	if (v->printmode == PRINTMODE_NONE) {
+		return;
+	}
+	xor = ((ulong)adr1) ^ ((ulong)adr2);
+	print_err(adr1, good, bad, xor);
 }
 
 /*
@@ -1218,104 +1197,130 @@ void ad_err1(ulong adr1, ulong adr2, ulong good, ulong bad)
  * Since this type of address error can also report data errors go
  * ahead and generate BadRAM patterns.
  */
-void ad_err2(ulong adr, ulong bad)
+void ad_err2(ulong *adr, ulong bad)
 {
 	int patnchg;
 
 	/* Process the address in the pattern administration */
 	patnchg=insertaddress ((ulong) adr);
 
+	update_err_counts();
 	if (v->printmode == PRINTMODE_ADDRESSES) {
-		print_err(adr, adr, bad, adr ^ bad);
-	} else {
-		/* printmode == PRINTMODE_PATTERNS */
-
-		dprint(LINE_INFO, COL_ERR, ++(v->ecount), 6, 0);
-		tseq[v->test].errors++;
-
+		print_err(adr, (ulong)adr, bad, ((ulong)adr) ^ bad);
+	} else if (v->printmode == PRINTMODE_PATTERNS) {
+		print_err_counts();
 		if (patnchg) { 
 			printpatn();
 		}
 	}
 }
-void print_hdr()
+void print_hdr(void)
 {
-		cprint(LINE_HEADER, 0,  "Tst  Pass   Failing Address      Good       Bad    Err-Bits  Count");
-		cprint(LINE_HEADER+1, 0,"---  ----  -------------------  --------  --------  --------  -----");
+	static int header = -1;
+	if (header == 1) {
+		return;
+	}
+	header = 1;
+	cprint(LINE_HEADER, 0,  "Tst  Pass   Failing Address          Good       Bad     Err-Bits  Count Chan");
+	cprint(LINE_HEADER+1, 0,"---  ----  -----------------------  --------  --------  --------  ----- ----");
 }
 
-/*
- * Print an individual error
- */
-void print_err( ulong *adr, ulong good, ulong bad, ulong xor) {
+static void update_err_counts(void)
+{
+	++(v->ecount);
+	tseq[v->test].errors++;
+}
+
+static void print_err_counts(void)
+{
+	dprint(LINE_INFO, COL_ERR, v->ecount, 6, 0);
+	dprint(LINE_INFO, COL_ECC_ERR, v->ecc_ecount, 6, 0);
+}
+
+static void common_err(ulong page, ulong offset)
+{
 	ulong mb;
 
 	/* Check for keyboard input */
 	check_input();
 
-	dprint(LINE_INFO, COL_ERR, ++(v->ecount), 6, 0);
-	tseq[v->test].errors++;
+	print_err_counts();
 
-	if (header == 0) {
-		header++;
-		print_hdr();
-	}
+	print_hdr();
 
 	scroll();
-
-	ecount = 1;
-	mb = (ulong)adr/1048576;
+	mb = page >> 8;
 	dprint(v->msg_line, 0, v->test, 3, 0);
 	dprint(v->msg_line, 4, v->pass, 5, 0);
-	hprint(v->msg_line, 11, (ulong)adr);
-	cprint(v->msg_line, 19, " -     . MB");
-	dprint(v->msg_line, 22, mb, 4, 0);
-	dprint(v->msg_line, 27, ((ulong)adr-(mb*1048576))/104858, 1, 0);
-	hprint(v->msg_line, 32, good);
-	hprint(v->msg_line, 42, bad);
-	hprint(v->msg_line, 52, xor);
-	dprint(v->msg_line, 62, ecount, 5, 0);
+	hprint(v->msg_line, 11, page);
+	hprint2(v->msg_line, 19, offset, 3);
+	cprint(v->msg_line, 22, " -      . MB");
+	dprint(v->msg_line, 25, mb, 5, 0);
+	dprint(v->msg_line, 31, ((page & 0xF)*10)/16, 1, 0);
+}
+/*
+ * Print an individual error
+ */
+void print_err( ulong *adr, ulong good, ulong bad, ulong xor) 
+{
+	ulong page, offset;
+
+	page = page_of(adr);
+	offset = ((unsigned long)adr) & 0xFFF;
+	common_err(page, offset);
+
+	ecount = 1;
+	hprint(v->msg_line, 36, good);
+	hprint(v->msg_line, 46, bad);
+	hprint(v->msg_line, 56, xor);
+	dprint(v->msg_line, 66, ecount, 5, 0);
 	v->eadr = adr;
 	v->exor = xor;
 }
 
+/*
+ * Print an ecc error
+ */
+void print_ecc_err(unsigned long page, unsigned long offset, 
+	int corrected, unsigned char syndrom, int channel)
+{
+	update_err_counts();
+	++(v->ecc_ecount);
+	if (v->printmode == PRINTMODE_NONE) {
+		return;
+	}
+	common_err(page, offset);
+
+	cprint(v->msg_line, 36, 
+		corrected?"corrected           ": "uncorrected         ");
+	hprint2(v->msg_line, 56, syndrom, 2);
+	cprint(v->msg_line, 66, "ECC"); 
+	dprint(v->msg_line, 71, channel, 2, 0);
+}
+
 #ifdef PARITY_MEM
 /*
  * Print a parity error message
  */
-void parity_err( ulong edi, ulong esi) {
-	ulong addr;
-	ulong mb;
-
-	/* Check for keyboard input */
-	check_input();
-
-	tseq[v->test].errors++;
-	dprint(LINE_INFO, COL_ERR, ++(v->ecount), 6, 0);
-
-	scroll();
-
-	if (header == 0) {
-		header++;
-		print_hdr();
-	}
+void parity_err( unsigned long edi, unsigned long esi) 
+{
+	unsigned long addr;
 
 	if (v->test == 5) {
 		addr = esi;
 	} else {
 		addr = edi;
 	}
-	mb = (ulong)addr/1048576;
-	dprint(v->msg_line, 0, v->test, 3, 0);
-	dprint(v->msg_line, 4, v->pass, 5, 0);
-	hprint(v->msg_line, 11, (ulong)addr);
-	cprint(v->msg_line, 19, " -     . MB");
-	dprint(v->msg_line, 22, mb, 4, 0);
-	dprint(v->msg_line, 27, ((ulong)addr-(mb*1048576))/104858, 1, 0);
-	cprint(v->msg_line, 32, "Parity error detected                ");
+	update_err_counts();
+	if (v->printmode == PRINTMODE_NONE) {
+		return;
+	}
+	common_err(page_of((void *)addr), addr & 0xFFF);
+	cprint(v->msg_line, 36, "Parity error detected                ");
 }
 #endif
 
+
 /*
  * Print the pattern array as a LILO boot option addressing BadRAM support.
  */
@@ -1354,11 +1359,14 @@ void printpatn (void)
 /*
  * Show progress by displaying elapsed time and update bar graphs
  */
-void do_tick()
+void do_tick(void)
 {
 	int i, pct;
 	ulong h, l, t;
 
+	/* FIXME only print serial error messages from the tick handler */
+	print_err_counts();
+	
 	nticks++;
 	v->total_ticks++;
 
@@ -1411,4 +1419,7 @@ void do_tick()
 
 	/* Check for keyboard input */
 	check_input();
+
+	/* Poll for ECC errors */
+	poll_errors();
 }
