@@ -1,4 +1,4 @@
-/* test.h - MemTest-86  Version 2.9
+/* test.h - MemTest-86  Version 3.0
  *
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady, cbrady@sgi.com
@@ -10,8 +10,30 @@
 #define E820NR  0x08           /* # entries in E820MAP */
 #define E820MAP 0x0c           /* our map */
 #define E820MAX 32             /* number of entries in E820MAP */
+#define E820ENTRY_SIZE 20
+#define MEMINFO_SIZE 0x28c
 
 #ifndef __ASSEMBLY__
+
+#define E820_RAM        1
+#define E820_RESERVED   2
+#define E820_ACPI       3 /* usable as RAM once ACPI tables have been read */
+#define E820_NVS        4
+
+struct e820entry {
+        unsigned long long addr;        /* start of memory segment */
+        unsigned long long size;        /* size of memory segment */
+        unsigned long type;             /* type of memory segment */
+};
+
+struct mem_info_t {
+	unsigned long e88_mem_k;	/* 0x00 */
+	unsigned long e801_mem_k;	/* 0x04 */
+	unsigned long e820_nr;		/* 0x08 */
+	struct e820entry e820[E820MAX];	/* 0x0c */
+					/* 0x28c */
+};
+
 typedef unsigned long ulong;
 #define SPINSZ		0x800000
 #define DEFTESTS	7
@@ -24,27 +46,32 @@ typedef unsigned long ulong;
 #define SCREEN_ADR	0xb8000
 
 #define TITLE_WIDTH	28
-#define LINE_TIME	8
+#define LINE_TIME	9
 #define COL_TIME	0
 #define LINE_TST	2
 #define LINE_RANGE	3
 #define LINE_CPU	1
 #define COL_MID		30
-#define LINE_INFO	8
+#define LINE_PAT        4
+#define COL_PAT		41
+#define LINE_INFO	9
+#define COL_CACHE_TOP   13
+#define COL_RESERVED    22
 #define COL_MMAP	29
 #define COL_CACHE	40
-#define COL_PAT		48
-#define COL_TST		60
-#define COL_PASS	64
-#define COL_ERR		71
-#define LINE_HEADER	9
-#define LINE_SCROLL	12
+#define COL_ECC		46
+#define COL_TST		52
+#define COL_PASS	56
+#define COL_ERR		63
+#define COL_ECC_ERR	72
+#define LINE_HEADER	11
+#define LINE_SCROLL	13
 #define BAR_SIZE	(78-COL_MID-9)
 
 #define POP_W	30
 #define POP_H	13
 #define POP_X	(80-POP_W-2)/2
-#define POP_Y	9
+#define POP_Y	10
 #define NULL	0
 
 /* memspeed operations */
@@ -58,79 +85,71 @@ typedef unsigned long ulong;
 #define SZ_MODE_PROBE		3
 
 #define getCx86(reg) ({ outb((reg), 0x22); inb(0x23); })
-#define serial_echo_outb(v,a) outb((v),(a)+0x3f8)
-#define serial_echo_inb(a)    inb((a)+0x3f8)
-#define BOTH_EMPTY (UART_LSR_TEMT | UART_LSR_THRE)
-/* Wait for transmitter & holding register to empty */
-#define WAIT_FOR_XMITR \
- do { \
-       lsr = serial_echo_inb(UART_LSR); \
- } while ((lsr & BOTH_EMPTY) != BOTH_EMPTY)
-
+int memcmp(const void *s1, const void *s2, ulong count);
+void *memmove(void *dest, const void *src, ulong n);
+int query_linuxbios(void);
+int query_pcbios(void);
 int insertaddress(ulong);
 void printpatn(void);
 void printpatn(void);
 void itoa(char s[], int n); 
 void reverse(char *p);
-void serial_echo_init();
-void serial_echo_print(char *s);
-void ttyprint(int y, int x, char *s);
+void serial_echo_init(void);
+void serial_echo_print(const char *s);
+void ttyprint(int y, int x, const char *s);
 void ttyprintc(int y, int x, char c);
-void cprint(int y,int x,char *s);
+void cprint(int y,int x, const char *s);
 void hprint(int y,int x,ulong val);
+void hprint2(int y,int x, ulong val, int len);
 void xprint(int y,int x,ulong val);
-void aprint(int y,int x,ulong val);
+void aprint(int y,int x,ulong page);
 void dprint(int y,int x,ulong val,int len, int right);
 void movinv1(int iter, ulong p1, ulong p2);
 void movinv32(int iter, ulong p1, ulong lb, ulong mb, int sval, int off);
 void modtst(int off, int iter, ulong p1, ulong p2);
 void error(ulong* adr, ulong good, ulong bad);
-void ad_err1(ulong adr1, ulong adr2, ulong good, ulong bad);
-void ad_err2(ulong adr, ulong bad);
-void do_tick();
-void init();
-void inter();
+void ad_err1(ulong *adr1, ulong *adr2, ulong good, ulong bad);
+void ad_err2(ulong *adr, ulong bad);
+void do_tick(void);
+void init(void);
+struct eregs;
+void inter(struct eregs *trap_regs);
 void set_cache(int val);
-void check_input();
-void footer();
-void scroll();
-void popup();
-void popdown();
-void popclear();
-void get_config();
-void get_printmode();
-void cpu_type();
-void addr_tst1();
-void addr_tst2();
-void block_move();
-void find_ticks();
-void print_err();
-void mem_size();
-void memsize_820();
-void memsize_801();
-void memsize_probe();
-void mem_size();
-void adj_mem();
-ulong getval(int x, int y);
-int get_key();
-int cpuspeed();
-ulong memspeed();
-int check_ram();
+void check_input(void);
+void footer(void);
+void scroll(void);
+void popup(void);
+void popdown(void);
+void popclear(void);
+void get_config(void);
+void get_printmode(void);
+void addr_tst1(void);
+void addr_tst2(void);
+void block_move(int iter);
+void find_ticks(void);
+void print_err(ulong *adr, ulong good, ulong bad, ulong xor);
+void print_ecc_err(ulong page, ulong offset, int corrected, 
+	unsigned char syndrom, int channel);
+void mem_size(void);
+void adj_mem(void);
+ulong getval(int x, int y, int result_shift);
+int get_key(void);
 int ascii_to_keycode(int in);
-void wait_keyup();
-void cacheable();
-void print_hdr();
-void restart();
+void wait_keyup(void);
+void print_hdr(void);
+void restart(void);
 void parity_err(ulong edi, ulong esi);
-void start_config();
-void clear_screen();
-void xmemspeed();
-int strlen();
-
-extern ulong idt_descr;
+void start_config(void);
+void clear_screen(void);
+void paging_off(void);
+int map_page(unsigned long page);
+void *mapping(unsigned long page_address);
+void *emapping(unsigned long page_address);
+unsigned long page_of(void *ptr);
 
 #define PRINTMODE_ADDRESSES 0
 #define PRINTMODE_PATTERNS  1
+#define PRINTMODE_NONE      2
 
 #define BADRAM_MAXPATNS 10
 
@@ -139,44 +158,30 @@ struct pair {
        ulong mask;
 };
 
-struct e820map {
-    int nr_map;
-    struct e820entry {
-        unsigned long long addr;        /* start of memory segment */
-        unsigned long long size;        /* size of memory segment */
-        unsigned long type;             /* type of memory segment */
-    } map[E820MAX];
-};
-int sanitize_e820_map(struct e820entry *a, struct e820entry *b, short nr,
-	 short res);
 
-#define E820_RAM        1
-#define E820_RESERVED   2
-#define E820_ACPI       3 /* usable as RAM once ACPI tables have been read */
-#define E820_NVS        4
-
-
-extern __inline__ void cache_off()
+static inline void cache_off(void)
 {
-        __asm__("push %eax\n\t"
+        asm(
+		"push %eax\n\t"
 		"movl %cr0,%eax\n\t"
                 "orl $0x40000000,%eax\n\t"  /* Set CD */
                 "movl %eax,%cr0\n\t"
-		".byte 0x0f,0x09\n\t"	/* Invalidate and flush cache */
+		"wbinvd\n\t"
 		"pop  %eax\n\t");
 }
-extern __inline__ void cache_on()
+static inline void cache_on(void)
 {
-        __asm__("push %eax\n\t"
+        asm(
+		"push %eax\n\t"
 		"movl %cr0,%eax\n\t"
                 "andl $0x9fffffff,%eax\n\t" /* Clear CD and NW */ 
                 "movl %eax,%cr0\n\t"
 		"pop  %eax\n\t");
 }
 
-extern __inline__ void reboot()
+static inline void reboot(void)
 {
-        __asm__(
+        asm(
 		"movl %cr0,%eax\n\t"
        		"andl  $0x00000011,%eax\n\t"
        		"orl   $0x60000000,%eax\n\t"
@@ -198,9 +203,14 @@ extern __inline__ void reboot()
 }
 
 struct mmap {
-	int v;
-	volatile ulong *start;
-	volatile ulong *end;
+	ulong pbase_addr;
+	ulong *start;
+	ulong *end;
+};
+
+struct pmap {
+	ulong start;
+	ulong end;
 };
 
 struct tseq {
@@ -222,30 +232,34 @@ struct cpu_ident {
 	char vend_id[12];
 	unsigned char cache_info[16];
 };
+#define X86_FEATURE_PAE		(0*32+ 6) /* Physical Address Extensions */
+
+#define MAX_MEM_SEGMENTS E820MAX
 
 /* Define common variables accross relocations of memtest86 */
 struct vars {
-	int firsttime;
 	int test;
 	int pass;
-	ulong *eadr;
-	ulong exor;
+	unsigned long *eadr;
+	unsigned long exor;
 	int msg_line;
 	int ecount;
+	int ecc_ecount;
 	int msegs;
 	int cache_flag;
 	int xtst_flag;
 	int testsel;
 	int scroll_start;
 	int rdtsc;
+	int pae;
 	int pass_ticks;
 	int total_ticks;
 	int pptr;
 	int tptr;
-	struct mmap map[E820MAX];
-	struct mmap rmap[E820MAX];
-	volatile ulong lim_lower;
-	volatile ulong lim_upper;
+	struct pmap pmap[MAX_MEM_SEGMENTS];
+	struct mmap map[MAX_MEM_SEGMENTS];
+	ulong plim_lower;
+	ulong plim_upper;
 	ulong clks_msec;
 	ulong starth;
 	ulong startl;
@@ -254,15 +268,19 @@ struct vars {
 	int printmode;
 	int numpatn;
 	struct pair patn [BADRAM_MAXPATNS];
-	char memsz_mode;
-	ulong test_mem;
-	ulong selected_mem;
-	ulong reserved;
-	ulong alt_mem_k;
-	ulong ext_mem_k;
-	short e820_nr;
-	struct e820entry e820[E820MAX];
+	ulong test_pages;
+	ulong selected_pages;
+	ulong reserved_pages;
 };
 
-#endif
+#define FIRMWARE_UNKNOWN   0
+#define FIRMWARE_PCBIOS    1
+#define FIRMWARE_LINUXBIOS 2
 
+extern struct vars * const v;
+extern unsigned char _start[], _end[], startup_32[];
+extern unsigned char _size, _pages;
+
+extern struct mem_info_t mem_info;
+
+#endif /* __ASSEMBLY__ */
